/**********************************************************
 *  	      STEPPER MOTOR WITH Micro-GT                 *
 *                     02/02/2017                         *
 *hardware Micro-GT mini ->unipolar steper motor          *
 *         interface: transistor array unipolar           *
 *         command buttons on pulldown                    *
 *         With serial/USB interface                      *
 *                   PIC16F876A                           *
 *          MpLabX     compiler Hitech C16                *
 *     performed by Marco Barbisan                     *
 *********************************************************/
#define _LEGACY_HEADERS
#include <pic.h>
#include "delay.h"
#include "settaggi.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <htc.h>

//#define stepper PORTB
#define aumenta RB6
#define diminuisci RB7
#define faseA RB0
#define faseB RB1
#define faseC RB2
#define faseD RB3

extern void Par_ser(void); //parser della stringa seriale che raggiunge il buffer
extern void trasmettiquota(int);
extern void salvaquota(int);
extern int leggiquota(void);
extern void avanza(void);
extern void indietreggia(void);

unsigned char RxPos,Flags,command;
unsigned char info[6]={0};
char fromUSB_avanti =0;
char fromUSB_indietro =0;
char speed=3;
int quota=0;

void main (){ //start main function body
    //Inizializzazione uart
    TXSTA= 0x20;		// TX abilitato
    RCSTA= 0X90;		// Registro RX USART
    BRGH = 0;		// USART High velocity
    SPBRG= 7;//15;		// BRGH = 0 Baud Rate Generator (15=19200 baud 20Mhz)

    //Inizializzazione TIMER1
    T1CON   = 0;		// TIMER 1 resettato
    T1CKPS1 = 0,T1CKPS0=0;	// TIMER1 prescaler = 1
    T1OSCEN = 0;		// TIMER1 external oscillator disable
    TMR1CS  = 0;		// TIMER1 internal clock (fosc/4)
    TMR1ON  = 0;		// TIMER1 in STOP

    //Inizializzazione interrupt
    INTCON = 0;		// Interrupt disabilitato
    PIE1   = 0;		// Interrupt di periferica disabilitati
    GIE    = 1;		// Global Interrupt Enable
    PEIE   = 1;		// Periferal Interrupt Enable
    RCIE   = 1;		// USART Receive Interrupt Enable

    RxPos=0;
    Flags=0;
    unsigned char manuale=0;

    settaggi();
    quota=leggiquota();
    while(1){ 
       if (Flags==1) Par_ser();
        
	   if(((aumenta==0) && (diminuisci==1))){ 
           avanza();
           manuale=1;
	   }
	   if((aumenta==1) && (diminuisci==0)){ 
           indietreggia();
           manuale=1;
	   }
	   
	   if(aumenta==1 && diminuisci==1){ 
          faseA=0;faseB=0;faseC=0;faseD=0; 
          if (manuale){
              salvaquota(quota);
              manuale=0;
          }
       }
       CLRWDT();   
    }
    
}

void trasmettiquota(int quota){
    char stringa[10]={0};
    int bah=0;
    bah=sprintf(stringa,"%d\n\r",quota);
    for(int i=0;i<9;i++){
        TXREG=stringa[i];
        while(!TRMT);
    }
}

void salvaquota(int quota){
    unsigned char a=(char)(quota & 0xFF);
    unsigned char b=(char)((quota >> 8) & 0xFF);
    eeprom_write(0,a);
    eeprom_write(1,b);
    DelayMs(4);
}

int leggiquota(){
    unsigned char a;
    unsigned char b;
    a=eeprom_read(0);
    b=eeprom_read(1);
    return a | b << 8;
}

void avanza(){
    faseA=0;faseB=0;faseC=0;faseD=1;
    DelayMs(speed);
    faseA=0;faseB=0;faseC=1;faseD=1;
    DelayMs(speed);
    faseA=0;faseB=0;faseC=1;faseD=0;
    DelayMs(speed);
    faseA=0;faseB=1;faseC=1;faseD=0;
    DelayMs(speed);
    faseA=0;faseB=1;faseC=0;faseD=0;
    DelayMs(speed);
    faseA=1;faseB=1;faseC=0;faseD=0;
    DelayMs(speed);
    faseA=1;faseB=0;faseC=0;faseD=0;
    DelayMs(speed);
    faseA=1;faseB=0;faseC=0;faseD=1;
    quota+=8;
}

void indietreggia(){
    faseA=1;faseB=0;faseC=0;faseD=1;
    DelayMs(speed);
    faseA=1;faseB=0;faseC=0;faseD=0;
    DelayMs(speed);
    faseA=1;faseB=1;faseC=0;faseD=0;
    DelayMs(speed);
    faseA=0;faseB=1;faseC=0;faseD=0;
    DelayMs(speed);
    faseA=0;faseB=1;faseC=1;faseD=0;
    DelayMs(speed);
    faseA=0;faseB=0;faseC=1;faseD=0;
    DelayMs(speed);
    faseA=0;faseB=0;faseC=1;faseD=1;
    DelayMs(speed);
    faseA=0;faseB=0;faseC=0;faseD=1;
    quota-=8;
}


void interrupt SerChar(void)    //prototipo ricezione
{
  if (RCREG==0x40 || RxPos > 10)  //se arriva la chiocciola dimensiona la stringa
  {
    for(int i=0;i<5;i++) info[i]=0;//azzero la stringa
    RxPos=0,Flags=0;
    command=0;
  }
  if (RxPos==1){
      command=RCREG;
      if (RCREG==0x0d) Flags=1;
  } 
  if (RxPos>1){
      if ((RCREG != 0x0d) && (RxPos < 8)){
        info[RxPos-2]=RCREG;
      } else{
        Flags=1;
      }
  }
  RxPos++;
}

void Par_ser(){		       // Analisi comandi
    if (quota != leggiquota()) salvaquota(quota);
    if (command==0x51){
        //Q:richiesta di quota
        trasmettiquota(quota);
    }
    if (command==0x53){
        //S: imposta la quota a un nuovo valore
        int nuovo=atoi(info);
        quota=nuovo;
        salvaquota(nuovo);
        trasmettiquota(nuovo);
    }
    if (command==0x4d){
        //M: muove il motore ad una data quota
        int desiderata;
        desiderata=atoi(info);
        while(desiderata > quota){
            avanza();
        }
        while(desiderata < quota){
            indietreggia();
        }
        salvaquota(quota);
        trasmettiquota(quota);
    }
    Flags=0;
}
